@@ -1,6 +1,6 @@
 import math
 from abc import ABC
-from typing import Callable, Union
+from typing import Callable, Union, Tuple, List
 
 import torch
 from transformers import CLIPTokenizer, CLIPTextModel
@@ -9,21 +9,31 @@ __all__ = ["EmbeddingsProvider"]
 
 
 class BaseTextualInversionManager(ABC):
-    def expand_textual_inversion_token_ids_if_necessary(self, token_ids: list[int]) -> list[int]:
+    def expand_textual_inversion_token_ids_if_necessary(self, token_ids: List[int]) -> List[int]:
         raise NotImplementedError()
 
 
 class EmbeddingsProvider:
 
     def __init__(self,
-                tokenizer: CLIPTokenizer, # converts strings to lists of int token ids
-                text_encoder: CLIPTextModel, # convert a list of int token ids to a tensor of embeddings
+                tokenizer: CLIPTokenizer,
+                text_encoder: CLIPTextModel,
                 textual_inversion_manager: BaseTextualInversionManager = None,
                 dtype_for_device_getter: Callable[[torch.device], torch.dtype] = lambda device: torch.float32,
+                truncate: bool = True
                 ):
+        """
+        `tokenizer`: converts strings to lists of int token ids
+        `text_encoder`: convert lists of token ids to embedding tensors
+        `textual_inversion_manager`: manage token insertion for textual inversions with vector length >1
+        `dtype_for_device_getter`: callback that returns an appropriate dtype for the requested device. if unset, defaults to torch.float32.
+        `truncate`: if True, truncate inputs to the maximum length specified by the tokenizer. if False, returns
+                    tensors that may be longer than the maximum length (but will always be an integer multiple of maximum length)
+        """
         self.tokenizer = tokenizer
         self.text_encoder = text_encoder
         self.textual_inversion_manager = textual_inversion_manager
+        self.truncate_to_model_max_length = truncate
 
         # by default always use float32
         self.get_dtype_for_device = dtype_for_device_getter
@@ -35,7 +45,7 @@ class EmbeddingsProvider:
 
 
     @classmethod
-    def apply_embedding_weights(cls, embeddings: torch.Tensor, per_embedding_weights: list[float],
+    def apply_embedding_weights(cls, embeddings: torch.Tensor, per_embedding_weights: List[float],
                                 normalize: bool) -> torch.Tensor:
         per_embedding_weights = torch.tensor(per_embedding_weights, dtype=embeddings.dtype, device=embeddings.device)
         if normalize:
@@ -47,11 +57,11 @@ class EmbeddingsProvider:
         return blended_embeddings
 
     def get_embeddings_for_weighted_prompt_fragments(self,
-                                                     text_batch: list[list[str]],
-                                                     fragment_weights_batch: list[list[float]],
+                                                     text_batch: List[List[str]],
+                                                     fragment_weights_batch: List[List[float]],
                                                      should_return_tokens: bool = False,
                                                      device='cpu',
-                                 ) -> Union[torch.Tensor, tuple[torch.Tensor, torch.Tensor]]:
+                                 ) -> Union[torch.Tensor, Tuple[torch.Tensor, torch.Tensor]]:
         """
 
         :param text_batch: A list of fragments of text to which different weights are to be applied.
@@ -136,7 +146,7 @@ class EmbeddingsProvider:
         else:
             return batch_z
 
-    def get_token_ids(self, texts: list[str], include_start_and_end_markers: bool = True) -> list[list[int]]:
+    def get_token_ids(self, texts: List[str], include_start_and_end_markers: bool = True) -> List[List[int]]:
         """
         Convert a list of strings like `["a cat", "a dog", "monkey riding a bicycle"]` into a list of lists of token
         ids like `[[bos, 0, 1, eos], [bos, 0, 2, eos], [bos, 3, 4, 0, 5, eos]]`. bos/eos markers are skipped if
@@ -152,9 +162,7 @@ class EmbeddingsProvider:
         # (part of `transformers` lib)
         token_ids_list = self.tokenizer(
             texts,
-            truncation=True,
-            max_length=self.max_token_count,
-            return_overflowing_tokens=False,
+            truncation=False,
             padding='do_not_pad',
             return_tensors=None,  # just give me lists of ints
         )['input_ids']
@@ -166,8 +174,7 @@ class EmbeddingsProvider:
             # pad for textual inversions with vector length >1
             if self.textual_inversion_manager is not None:
                 token_ids = self.textual_inversion_manager.expand_textual_inversion_token_ids_if_necessary(token_ids)
-            # truncate if necessary to max_length-2 (leaving room for bos/eos)
-            token_ids = token_ids[0:self.max_token_count - 2]
+
             # add back eos/bos if requested
             if include_start_and_end_markers:
                 token_ids = [self.tokenizer.bos_token_id] + token_ids + [self.tokenizer.eos_token_id]
@@ -176,7 +183,7 @@ class EmbeddingsProvider:
 
         return result
 
-    def get_token_ids_and_expand_weights(self, fragments: list[str], weights: list[float], device: str) -> (torch.Tensor, torch.Tensor):
+    def get_token_ids_and_expand_weights(self, fragments: List[str], weights: List[float], device: str) -> (torch.Tensor, torch.Tensor):
         '''
         Given a list of text fragments and corresponding weights: tokenize each fragment, append the token sequences
         together and return a padded token sequence starting with the bos marker, ending with the eos marker, and padded
@@ -196,62 +203,96 @@ class EmbeddingsProvider:
             fragments = ['']
             weights = [1.0]
         per_fragment_token_ids = self.get_token_ids(fragments, include_start_and_end_markers=False)
-        all_token_ids = []
-        per_token_weights = []
+        all_token_ids: List[int] = []
+        all_token_weights: List[float] = []
         #print("all fragments:", fragments, weights)
         for this_fragment_token_ids, weight in zip(per_fragment_token_ids, weights):
             # append
             all_token_ids += this_fragment_token_ids
             # fill out weights tensor with one float per token
-            per_token_weights += [float(weight)] * len(this_fragment_token_ids)
+            all_token_weights += [float(weight)] * len(this_fragment_token_ids)
 
-        # leave room for bos/eos
-        max_token_count_without_bos_eos_markers = self.max_token_count - 2
-        if len(all_token_ids) > max_token_count_without_bos_eos_markers:
-            excess_token_count = len(all_token_ids) - max_token_count_without_bos_eos_markers
-            # TODO build nice description string of how the truncation was applied
-            # this should be done by calling self.tokenizer.convert_ids_to_tokens() then passing the result to
-            # self.tokenizer.convert_tokens_to_string() for the token_ids on each side of the truncation limit.
-            print(f">> Prompt is {excess_token_count} token(s) too long and has been truncated")
-            all_token_ids = all_token_ids[0:max_token_count_without_bos_eos_markers]
-            per_token_weights = per_token_weights[0:max_token_count_without_bos_eos_markers]
+        return self._chunk_and_pad_token_ids(all_token_ids, all_token_weights, device=device)
 
-        # pad out to a self.max_length-entry array: [bos_token, <prompt tokens>, eos_token, pad_token...]
-        # (typically self.max_length == 77)
-        all_token_ids = [self.tokenizer.bos_token_id] + all_token_ids + [self.tokenizer.eos_token_id]
-        per_token_weights = [1.0] + per_token_weights + [1.0]
-        pad_length = self.max_token_count - len(all_token_ids)
-        all_token_ids += [self.tokenizer.pad_token_id] * pad_length
-        per_token_weights += [1.0] * pad_length
+    def _chunk_and_pad_token_ids(self, token_ids: List[int], token_weights: List[float], device: str
+                                 ) -> Tuple[torch.Tensor, torch.Tensor]:
+
+        remaining_token_ids = token_ids
+        remaining_token_weights = token_weights
+        chunk_length_without_eos_bos_markers = self.max_token_count - 2
+
+        all_token_ids = []
+        all_token_weights = []
+        while True:
+            # each chunk must leave room for bos/eos
+            chunk_token_ids = remaining_token_ids[0:chunk_length_without_eos_bos_markers]
+            chunk_token_weights = remaining_token_weights[0:chunk_length_without_eos_bos_markers]
+            # update remaining
+            remaining_token_ids = remaining_token_ids[chunk_length_without_eos_bos_markers:]
+            remaining_token_weights = remaining_token_weights[chunk_length_without_eos_bos_markers:]
+
+            # pad out to a self.max_length-entry array: [eos_token, <prompt tokens>, eos_token[, pad_token, ...]]
+            # (typically self.max_length == 77)
+            chunk_token_ids = [self.tokenizer.bos_token_id] + chunk_token_ids + [self.tokenizer.eos_token_id]
+            chunk_token_weights = [1.0] + chunk_token_weights + [1.0]
+            pad_length = self.max_token_count - len(chunk_token_ids)
+            chunk_token_ids += [self.tokenizer.pad_token_id] * pad_length
+            chunk_token_weights += [1.0] * pad_length
+
+            all_token_ids += chunk_token_ids
+            all_token_weights += chunk_token_weights
+
+            if self.truncate_to_model_max_length or len(remaining_token_ids) == 0:
+                break
 
         all_token_ids_tensor = torch.tensor(all_token_ids, dtype=torch.long, device=device)
-        per_token_weights_tensor = torch.tensor(per_token_weights,
-                                                dtype=self.get_dtype_for_device(self.text_encoder.device),
-                                                device=device)
+        all_per_token_weights_tensor = torch.tensor(all_token_weights,
+                                                    dtype=self.get_dtype_for_device(self.text_encoder.device),
+                                                    device=device)
         #print(f"assembled all_token_ids_tensor with shape {all_token_ids_tensor.shape}")
-        return all_token_ids_tensor, per_token_weights_tensor
+        return all_token_ids_tensor, all_per_token_weights_tensor
 
 
     def build_weighted_embedding_tensor(self, token_ids: torch.Tensor, per_token_weights: torch.Tensor) -> torch.Tensor:
         """
         Build a tensor that embeds the passed-in token IDs and applies the given per_token weights
-        :param token_ids: A tensor of shape `[self.max_length]` containing token IDs (ints)
-        :param per_token_weights: A tensor of shape `[self.max_length]` containing weights (floats)
-        :return: A tensor of shape `[1, self.max_length, token_dim]` representing the requested weighted embeddings
-        where `token_dim` is 768 for SD1 and 1280 for SD2.
-        """
-        # print(f"building weighted embedding tensor for {tokens} with weights {per_token_weights}")
-        if token_ids.shape != torch.Size([self.max_token_count]):
-            raise ValueError(f"token_ids has shape {token_ids.shape} - expected [{self.max_token_count}]")
 
-        z = self.text_encoder(token_ids.unsqueeze(0), return_dict=False)[0]
+        :param token_ids: A tensor of shape `n*[self.max_length]` containing token IDs (ints) where n is some arbitrary
+            integer (i.e. n==1 for shorter prompts, or it may be >1 if there are more than max_length tokens in the original prompt)
+        :param per_token_weights: A tensor containing weights (floats), with the same shape as token_ids
+
+        :return: A tensor of shape `[1, token_ids.shape[0], token_dim]` representing the requested weighted embeddings
+            where `token_dim` is 768 for SD1 and 1280 for SD2.
+        """
+        # print(f"building weighted embedding tensor for {tokens} with weights {token_weights}")
+        if token_ids.shape[0] % self.max_token_count != 0:
+            raise ValueError(f"token_ids has shape {token_ids.shape} - expected a multiple of {self.max_token_count}")
+
+        chunk_start_index = 0
         empty_token_ids = torch.tensor([self.tokenizer.bos_token_id] +
                                        [self.tokenizer.eos_token_id] +
                                        [self.tokenizer.pad_token_id] * (self.max_token_count - 2),
-                                       dtype=torch.int, device=z.device).unsqueeze(0)
+                                       dtype=torch.int, device=self.text_encoder.device).unsqueeze(0)
         empty_z = self.text_encoder(empty_token_ids, return_dict=False)[0]
-        batch_weights_expanded = per_token_weights.reshape(per_token_weights.shape + (1,)).expand(z.shape).to(z)
-        z_delta_from_empty = z - empty_z
-        weighted_z = empty_z + (z_delta_from_empty * batch_weights_expanded)
+        weighted_z = None
+
+        chunk_size = self.max_token_count
+        while chunk_start_index < token_ids.shape[0]:
+            next_chunk_start_index = chunk_start_index+chunk_size
+            chunk_token_ids = token_ids[chunk_start_index:next_chunk_start_index]
+            chunk_per_token_weights = per_token_weights[chunk_start_index:next_chunk_start_index]
+
+            z = self.text_encoder(chunk_token_ids.unsqueeze(0), return_dict=False)[0]
+            batch_weights_expanded = chunk_per_token_weights.reshape(
+                chunk_per_token_weights.shape + (1,)).expand(z.shape).to(z)
+
+            z_delta_from_empty = z - empty_z
+            this_weighted_z = empty_z + (z_delta_from_empty * batch_weights_expanded)
+            weighted_z = (
+                this_weighted_z
+                if weighted_z is None
+                else torch.cat([weighted_z, this_weighted_z], dim=1)
+            )
+            chunk_start_index += chunk_size
 
         return weighted_z
